var sock = [{},{},{},{}];
var mode = 0; //0 normal; 1 setting

var boot;
var heartbeat;
var outon;
var outoff;
var inon;
var inoff;

var cfg;
function loadCfg()
{
	var f = File.open("1");
	var l = File.len(f);
	if(l > 0)
	{
		var s = File.read(f, l).gets();
		cfg = JSON.parse(s);
	}

	if(!cfg)
	{
		cfg = {};
		cfg.rsopt = 1;
		cfg.br232 = 115200;
		cfg.br485 = 9600;
		cfg.addr0 = "iot.easelive.cn";
		cfg.port0 = 10808;
		File.write(f, JSON.stringify(cfg));
	}
	File.close(f);
}

function saveCfg(d)
{
	var f = File.open("1");
	var c = {};
	for(var k in d) if(k != "cmd" && k != "sn") c[k] = d[k];
	File.write(f, JSON.stringify(c));
	File.close(f);
}

function encodePacket(s)
{
	var c = Coder.crc16(s);
	var pk = Bytes.create(s.length + 5);
	pk.seti(0xEA);
	pk.seti(s.length, 1, "2b");
	pk.sets(s, 3);
	pk.seti(c, 3 + s.length, "2b");
	return pk;
}

function trace(s)
{
	if(cfg.trace)
	{
		var cmd = {cmd: 255, trace: s};
		UART.send(1, encodePacket(JSON.stringify(cmd)));
	}
}

function sendSetInfo()
{
	var cmd = {cmd: 1, sn: Box.sn()};
	for(var k in cfg) cmd[k] = cfg[k];
	UART.send(1, encodePacket(JSON.stringify(cmd)));
}

function sendResult(c, r)
{
	var cmd = {cmd:c,result:r};
	UART.send(1, encodePacket(JSON.stringify(cmd)));
}

function handleSetCmd(s)
{
	trace(s);
	var d = JSON.parse(s);
	if(d == undefined)
		return;

	switch(d["cmd"])
	{
		case 1:
			break;
		case 2:
			saveCfg(d);
			sendResult(2, 0);
			Box.reset();
			break;		
	}
}

var buf;
var index = 0;
var len = 0;
var crc = 0;
var state = 0;
function handleUartRecv(d)
{
	var i;
	for(i = 0; i < d.length; i++)
	{
		var b = d.geti(i);
		switch(state)
		{
			case 0:
				if(b == 0xEA)
					state++;
				break;
			case 1:
				len = b;
				state++;
				break;
			case 2:
				len <<= 8;
				len += b;
				if(len > 2048)
				{
					state = 0;
					return;
				}
				else
				{
					state++;
					index = 0;
				}
				break;
			case 3:
				buf.seti(b, index);
				index++;
				if(index >= len)
					state++;
				break;
			case 4:
				buf.seti(b, index);
				index++;
				state++;
				break;
			case 5:
				buf.seti(b, index);
				index++;
				if(Coder.crc16(buf.sub(0, index)) == 0)
					handleSetCmd(buf.gets(0, index - 2));
				state = 0;
				break;
		}
	}
}

function sendBoot(n)
{
	if(cfg["addr" + n] == "iot.easelive.cn")
	{
		var cmd = {cmd:1,ver:Box.ver(),model:3};
		cmd.sn = Box.sn();
		cmd.devid = Box.ccid();
		Net.send(n, encodePacket(JSON.stringify(cmd)));
	}
	else if(boot && boot.length > 0)
	{
		Net.send(n, boot);
	}
}

function sendHeartbeat(n)
{
	if(heartbeat && heartbeat.length > 0)
	{
		Net.send(n, heartbeat);
	}
}

function sendData(data)
{
	for(var i = 0; i < 4; i++)
	{
		if(sock[i].enabled)
		{
			sock[i].sendbuf[sock[i].sendbuf.length] = data;
			if(!sock[i].issending)
			{
				sock[i].hbcounter = cfg.hbperiod;
				sock[i].issending = true;
				Net.send(i, sock[i].sendbuf[0]);
			}
		}
	}
}

function handleOutTrigger(data)
{
	if(outon == data)
	{
		GPIO.set(GPIO.OUT1, 1);
		GPIO.set(GPIO.DATA, 1);
		return true;
	}
	if(outoff == data)
	{
		GPIO.set(GPIO.OUT1, 0);
		GPIO.set(GPIO.DATA, 0);
		return true;
	}
	return false;
}


function handleNetEvent(m)
{
	switch(m.event)
	{
		case Net.READY:
			for(var i = 0; i < 4; i++)
			{
				if(sock[i].enabled)
				{
					Net.connect(i, sock[i].addr, sock[i].port);
				}
			}
			break;
		case Net.CONN_OK:
			sendBoot(m.id);
			sock[m.id].hbcounter = cfg.hbperiod;
			if(sock[m.id].sendbuf.length > 0)
				Timer.start(2, 1000);
			break;
		case Net.CONN_FAIL:
			Net.connect(m.id, sock[m.id].addr, sock[m.id].port);
			break;
		case Net.CONN_CLOSE:
			Timer.stop(2);
			Net.connect(m.id, sock[m.id].addr, sock[m.id].port);
			break;
		case Net.RECV:
			if(!handleOutTrigger(m.data))
			{
				if(cfg.rsopt == 1 || cfg.rsopt == 3)
					UART.send(1, m.data);
				if(cfg.rsopt == 2 || cfg.rsopt == 3)
					UART.send(2, m.data);
			}
			break;
		case Net.SEND_OK:
			sock[m.id].issending = false;
			sock[m.id].retry = 0;
			sock[m.id].sendbuf.splice(0, 1);
			Timer.start(2, 500);
			break;
		case Net.SEND_FAIL:
			sock[m.id].issending = false;
			sock[m.id].retry++;
			if(sock[m.id].retry >= 3)
			{

				sock[m.id].retry = 0;
				sock[m.id].sendbuf.splice(0, 1);
			}
			Timer.start(2, 500);
			break;
	}
}

var rs232_inited = false;
function handleTimerOut(id)
{
	switch(id)
	{
		case 1: //send heartbeat packet
			{
				if(!rs232_inited)
				{
					rs232_inited = true;
					if(cfg.rsopt == 1 || cfg.rsopt == 3)
						if(cfg.br232 != 115200)
							UART.open(1, cfg.br232);
				}
				for(var i = 0; i < 4; i++)
				{
					if(sock[i].hbcounter)
					{
						sock[i].hbcounter--;
						if(sock[i].hbcounter == 0)
						{
							sendHeartbeat(i);
							sock[i].hbcounter = cfg.hbperiod;
						}
					}
				}
				Timer.start(1, 1000);
			}
			break;
		case 2: //send data come from uart
			{
				for(var i = 0; i < 4; i++)
				{
					if(sock[i].enabled && !sock[i].issending && sock[i].sendbuf.length)
					{
						sock[i].issending = true;
						sock[i].hbcounter = cfg.hbperiod;
						Net.send(i, sock[i].sendbuf[0]);
					}
				}
			}
			break;
	}
}

function hex(c)
{
	if(c >= 0x41 && c <= 0x46)
		return c - 0x41 + 10;
	if(c >= 0x61 && c <= 0x66)
		return c - 0x61 + 10;
	if(c >= 0x30 && c <= 0x39)
		return c - 0x30;
	return -1;
}
function parse(s)
{
	var b = Bytes.create(s.length + 100);
	var j = 0;
	for(var i = 0; i < s.length; i++)
	{
		if(s[i] == "\\")
		{
			i++;
			if(i < s.length)
			{
				var v = hex(s.charCodeAt(i));
				if(s[i] == "\\")
					b.seti(0x5c, j);
				else if(s[i] == "r")
					b.seti(0x0a, j);
				else if(s[i] == "n")
					b.seti(0x0d, j);
				else if(s[i] == "s")
				{
					var sn = Box.sn();
					b.sets(sn, j);
					j += sn.length - 1;
				}
				else if(s[i] == "m")
				{
					var imsi = Box.imsi();
					b.sets(imsi, j);
					j += imsi.length - 1;
				}
				else if(s[i] == "i")
				{
					var ccid = Box.ccid();
					b.sets(ccid, j);
					j += ccid.length - 1;
				}
				else if(v >= 0)
				{
					i++;
					if(i < s.length)
					{
						var v2 = hex(s.charCodeAt(i));
						if(v2 < 0)
							break;
						b.seti(v * 16 + v2, j);
					}
				}
				else
					break;
			}
			else
				break;
		}
		else
			b.seti(s.charCodeAt(i), j);
		j++;
	}
	return b.sub(0, j);
}

GPIO.set(GPIO.POWER, 1);

loadCfg();
if(cfg.boot && cfg.boot.length)
	boot = parse(cfg.boot);
if(cfg.heartbeat && cfg.heartbeat.length)
	heartbeat = parse(cfg.heartbeat);
if(cfg.outon && cfg.outon.length)
	outon = parse(cfg.outon);
if(cfg.outoff && cfg.outoff.length)
	outoff = parse(cfg.outoff);
if(cfg.inon && cfg.inon.length)
	inon = parse(cfg.inon);
if(cfg.inoff && cfg.inoff.length)
	inoff = parse(cfg.inoff);

for(var i = 0; i < 4; i++)
{
	sock[i].addr = cfg["addr" + i];
	sock[i].port = cfg["port" + i];
	sock[i].enabled = sock[i].addr && sock[i].port;
	sock[i].issending = false;
	sock[i].sendbuf = [];
	sock[i].retry = 0;
}
if(sock[0].enabled || sock[1].enabled || sock[2].enabled || sock[3].enabled)
	Net.init();
if(cfg.rsopt == 2 || cfg.rsopt == 3)
	UART.open(2, cfg.br485);
if(inon || inoff)
	GPIO.listen(GPIO.IN1);
//cfg.trace = true;
//trace("begin");
Timer.start(1, 1000);

while(true)
{
	var m = Event.get();
	switch(m.msg)
	{
		case Event.NET_EVENT:
			if(mode == 0)
				handleNetEvent(m);
			break;
		case Event.UART_RECV:
	 		if(m.port == 1)
	 		{
				if(m.data.length == 10 && m.data.gets() == "++DTUSET++")
				{
					mode = 1;
					buf = Bytes.create(2048)
					sendSetInfo();
				}
				else
				{
					if(mode == 0)
					{
						if(cfg.rsopt == 1 || cfg.rsopt == 3)
							sendData(m.data);
					}
					else
						handleUartRecv(m.data);
				}
	 		}
			if(m.port == 2 && mode == 0)
			{
				if(cfg.rsopt == 2 || cfg.rsopt == 3)
					sendData(m.data);
			}
			break;
		case Event.TIMER_OUT:
			if(mode == 0)
				handleTimerOut(m.id);
			break;
		case Event.PIO_CHANGE:
			if(inon && m.level == 1)
				sendData(inon);
			if(inoff && m.level == 0)
				sendData(inoff);
			break;
	}
}
